(function (factory) {
    if (typeof define === 'function' && define.amd) {
        // AMD
        define(['jquery', 'datatables.net'], function ($) {
            return factory($, window, document);
        });
    }
    else if (typeof exports === 'object') {
        // CommonJS
        module.exports = function (root, $) {
            if (!root) {
                root = window;
            }

            if (!$ || !$.fn.dataTable) {
                // Require DataTables, which attaches to jQuery, including
                // jQuery if needed and have a $ property so we can access the
                // jQuery object that is used
                $ = require('datatables.net')(root, $).$;
            }

            return factory($, root, root.document);
        };
    }
    else {
        // Browser
        factory(jQuery, window, document);
    }
}(function ($, window, document, undefined) {
    'use strict';
    var DataTable = $.fn.dataTable;


    /* Set the defaults for DataTables initialisation */
    $.extend(true, DataTable.defaults, {
        dom:
            "<'row'<'col-sm-6'l><'col-sm-6'f>>" +
            "<'row'<'col-sm-12'tr>>" +
            "<'row'<'col-sm-5'i><'col-sm-7'p>>",
        renderer: 'bootstrap'
    });


    /* Default class modification */
    $.extend(DataTable.ext.classes, {
        sWrapper: "dataTables_wrapper form-inline dt-bootstrap",
        sFilterInput: "form-control input-sm",
        sLengthSelect: "form-control input-sm",
        sProcessing: "dataTables_processing panel panel-default"
    });

    return DataTable;
}));

function GenerateLeftLine(marginLeft)
{
    return '<div style="height: 100%; position: absolute; width: 10px; margin-left: ' + marginLeft + 'px; margin-top: -9px; border-left: 1px dashed #ccc;">&nbsp;</div>'
}

function GenerateGuideLines()
{
    for (var i = 1; ; i++)
    {
        if (!GenerateGuideLinesInternal(i))
        {
            break;
        }
    }   
}

function GenerateGuideLinesInternal(position)
{
    var items = $('.item-level-' + position);

    if (items.length == 0)
        return false;

    var maxLevels = 14;
    var marginShift = 25;
    var marginInitial = -20;
    
    // limit the number
    if (position > maxLevels)
        position = maxLevels;

    var html = "";
    var margin = marginInitial;
    for (var i = 0; i < position ; i++) 
    {
        html = GenerateLeftLine(margin) + html;
        margin = margin - marginShift;
    }

    for (var i = 0; i < items.length; i++)
    {
        var item = $(items[i]); 
        item.html(html + item.html());
    }
    return true;
}

function FilterTable(jObj) {
    var jTable = $('#tableResults-' + jObj.attr('tid'));
    if (jObj.val() == "all") {
        jTable.DataTable().column(5).search("").draw();
    } else if (jObj.val() == "pass") {
        jTable.DataTable().column(5).search("pass").draw();
    } else if (jObj.val() == "fail") {
        jTable.DataTable().column(5).search("fail").draw();
    }
    else if (jObj.val() == "info") {
        jTable.DataTable().column(5).search("info").draw();
    }
    else if (jObj.val() == "warn") {
        jTable.DataTable().column(5).search("warn").draw();
    }
}

function dataToTable(dataset) {
    var html = '<label>Chart Values</label><table border="1" style="width: 100%; border: #e4e4e4 1px solid;">';
    html += '<thead><tr><th style="width:120px; background-color: #FAFAFA;"></th>';
    
    var columnCount = 0;
    jQuery.each(dataset.datasets, function (idx, item) {
        html += '<th style="background-color:' + item.backgroundColor + ';">' + item.label + '</th>';
        columnCount += 1;
    });

    html += '</tr></thead>';

    jQuery.each(dataset.labels, function (idx, item) {
        html += '<tr><td style="background-color: #FAFAFA">' + item + '</td>';
        for (i = 0; i < columnCount; i++) {
            html += '<td style="background-color:' + dataset.datasets[i].backgroundColor + ';">' + dataset.datasets[i].data[idx] + '</td>';
        }
        html += '</tr>';

        html += '<tr class="hiddenTrClass" style="display: none;"><td colspan="' + (columnCount + 1) + '" class="hiddenRow"></td>';
        html += '</tr>';
    });

    html += '</tr><tbody></table>';

    return html;
};

function unbindChartEvents() {
    jQuery('a[data-toggle="tab"]').unbind('shown.bs.tab');
    jQuery('.accordion-body').unbind('shown.bs.collapse');
}

function bindChartEvents() {
    jQuery('a[data-toggle="tab"]').on('shown.bs.tab', function (e) {
        var func = jQuery(e.target).attr('function');
        if (func != '') {
      	  window[func]();
          jQuery(e.target).attr('function', '');
        }
    });

    jQuery('.accordion-body').on('shown.bs.collapse', function () {
        var func = jQuery(this).attr('function');
        if (func != '') {
      	  window[func]();
          jQuery(this).attr('function', '');
          jQuery('a[function="' + func + '"]').attr('function', '');
        }
    });
}

$(document).ready(function () {
    GenerateGuideLines();
    var table = $('.data-table').DataTable({
        "bSort": false
    });

    $('#globalFilter').change(function () {
        var jObj = $(this);
        var jObjVal = jObj.val();
        
        $(".table-filter").each(function( index ) {
            var selectObj = $(this);
            selectObj.val(jObjVal);
            FilterTable(selectObj);
        });

        if (jObj.val() != "all") {
        	var nullTables = [];

	        $(".dataTables_empty").each(function( index ) {
	            var td = $(this);
	            var dt = td.closest(".dataTable");
	            nullTables.push(dt.attr("id"));
	            
	            var innerListObject = td.closest(".inner-list");
	            if (innerListObject.hasClass('in')) {
	            	innerListObject.collapse("hide");
	            }
	        });

	        $(".list-group-item").each(function( index ) {
	            var listItem = $(this);
	            if (!listItem.hasClass("item-level-0")){
					listItem.hide();
	            }
	        });

	        // find not empty tables
	        $(".dataTable").each(function( index ) {
	            var dt = $(this);
	            if (!nullTables.includes(dt.attr("id"))) {
	            	var listItem = dt.closest(".list-group-item");
	       			while (true)
	       			{
	       				if (listItem.hasClass("inner-list")) {
	       					if (!listItem.hasClass('in')) {
				            	listItem.collapse("show");
				            }

	       					listItem = listItem.prev();
	       				}
	       				
	       				listItem.show();
	       				if (listItem.hasClass("item-level-0")) {
	       					break;
	       				}

	       				listItem = listItem.parent();
	       				listItem.show();
	       			}
	            }
	            
	        });
	    }
	    else {
	    	$(".list-group-item").each(function( index ) {
	            var listItem = $(this);
	            listItem.show();
	        });
	    }
    });

    $(".table-filter").change(function () {
        var jObj = $(this);
        FilterTable(jObj);
    });

    $('.dataTables_filter').append('<button type="button" class="btn btn-default btn-sm collapse-rows" style="margin-left: 5px;"><span class="glyphicon glyphicon-collapse-down"></span> <span class="text">Expand/Collapse Rows</span></button>');

    $('#collapseall').on('click', function () {
        if ($('.inner-list:visible').length <= $('.inner-list:hidden').length)
            $('.inner-list').collapse('show');
        else
            $('.inner-list').collapse('hide');
    });

    $('.collapse-rows').on('click', function () {
        var id = $(this).parent('.dataTables_filter').attr('id');
        id = '#' + id.replace("_filter", "") + ' ';
        if ($(id + '.accordion-body:visible').length <= $(id + '.accordion-body:hidden').length)
            $(id + '.accordion-body').collapse('show');
        else
            $(id + '.accordion-body').collapse('hide');
    });

    $('.runtest-link').on('click', function () {
        var tableId = $(this).attr('tableId');
        var obj = $('#collapse-list-' + tableId);
        if (!obj.hasClass('in')) {
            obj.collapse('show');
            obj.on('shown.bs.collapse', function () {
                obj.off('shown.bs.collapse');
                jQuery.scrollTo(obj, 300, { offset: -200 });
            })
        } else {
            jQuery.scrollTo(obj, 300, { offset: -200 });
        }
    });

    var ctx = document.getElementById('chart').getContext('2d');
            
    var green = ctx.createLinearGradient(0, 0, 0, 150);//f8ffe8 0%,#e3f5ab 20%,#b7df2d
    green.addColorStop(0, 'green');   
    green.addColorStop(1, '#04cc04');

    var blue = ctx.createLinearGradient(0, 0, 0, 150);//f8ffe8 0%,#e3f5ab 20%,#b7df2d
    blue.addColorStop(0, '#333ece');   
    blue.addColorStop(1, '#8282e2');

    var red = ctx.createLinearGradient(0, 0, 0, 150);//f8ffe8 0%,#e3f5ab 20%,#b7df2d
    red.addColorStop(0, '#b90000');   
    red.addColorStop(1, '#ff5c5c');

    var orange = ctx.createLinearGradient(0, 0, 0, 150);//f8ffe8 0%,#e3f5ab 20%,#b7df2d
    orange.addColorStop(0, 'orange');   
    orange.addColorStop(1, '#f5c368');
  
    var config = {
      type: 'pie',
      data: {
        datasets: [{
          data: [
            parseInt($('#totalPassed').html()),
            parseInt($('#totalInfo').html()),
            parseInt($('#totalWarning').html()),
            parseInt($('#totalFailed').html()),
          ],
            backgroundColor: [
            green,
            blue,
            orange,
            red
          ],
          hoverBackgroundColor: [
            'green',
            '#333ece',
            'orange',
            '#b90000'
          ],
          borderWidth: [
            0,
            0,
            0,
            0
        ]}],
        labels: [
          'Passed',
          'Information',
          'Warning',
          'Failed'
        ]},
        options: {
          responsive: true,
          legend: {
          display: false
        }
      }
    };
    
    $('#simpleChartContainer').css('display', 'none');
    $('#canvasChartContainer').css('display', 'block');
  
    window.myPie = new Chart(ctx, config);
    
    // process statuses
    $('#totalPassed').html($('#totalPassed').html().replace('Passed', '<span style="color: green; font-weight: bold;">Passed</span>'));
    $('#totalInfo').html($('#totalInfo').html().replace('Information', '<span style="color: #333ece; font-weight: bold;">Information</span>'));
    $('#totalWarning').html($('#totalWarning').html().replace('Warning', '<span style="color: orange; font-weight: bold;">Warning</span>'));
    $('#totalFailed').html($('#totalFailed').html().replace('Failed', '<span style="color: #b90000; font-weight: bold;">Failed</span>'));

    bindChartEvents();
    
    jQuery('.data-table').on('page.dt', function () {
        unbindChartEvents();
    });

    jQuery('.data-table').on('draw.dt', function () {
        bindChartEvents();
    } );
});